import 'dart:convert';

import 'package:fis_common/event/event_type.dart';
import 'package:fis_jsonrpc/services/index.dart';

import 'package:vid/us/vid_us_image.dart';

/// Webview 控制器接口
abstract class IMeasure3DViewController {
  /// [Carotid] ✅Measure3DView 控制器接口

  /// 播放器模式更新事件
  late final FEventHandler<MeasureMode> updatePlayerMode;

  /// 壳子中 mdl 文件下载完成事件通知
  late final FEventHandler<void> onShellLoadedMdlFile;

  /// 检查记录ID
  String get recordId;
  set recordId(String value);

  ///检查记录3D数据资源信息
  List<Ultra3DResourceInfo> get ultra3DResourceInfoList;
  set ultra3DResourceInfoList(List<Ultra3DResourceInfo> value);

  /// 颈动脉合成结果
  CarotidResultDTO get carotidResult;
  set carotidResult(CarotidResultDTO value);

  ///是否存在3D数据
  bool get exist3DData;
  set exist3DData(bool value);

  ///是否允许3D转2D测量
  bool get enable2DMeasure;
  set enable2DMeasure(bool value);

  /// 用于2D测量的截面图像
  VidUsImage? get image4Measure;
  set image4Measure(VidUsImage? value);

  /// 当前模式
  MeasureMode get curMeasureMode;
  set curMeasureMode(MeasureMode value);
}

/// 测量模式
enum MeasureMode {
  /// vid 播放器模式
  vidMode,

  /// 颈动脉 3D 测量模式
  carotid3DMode,

  /// 颈动脉 2D 测量模式
  carotid2DMode,
}

///Shell 定义的数据格式
class Ultra3DResourceInfo {
  Ultra3DResourceInfo(CarotidResultDTO dto) {
    dataId = dto.mdlFile!.split("/").last.split(".")[0];
    surfaceUrl = dto.surfaceFile;
    mdFileUrl = dto.mdlFile;
    carotidScanType = dto.carotidScanType;
    carotidScanDirection = dto.carotidScanDirection;
  }

  String? dataId;
  String? surfaceUrl;
  String? mdFileUrl;
  CarotidScanTypeEnum carotidScanType = CarotidScanTypeEnum.CarotidLeft;
  CarotidScanDirectionEnum carotidScanDirection =
      CarotidScanDirectionEnum.TopToBottom;

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    if (dataId != null) map['DataId'] = dataId;
    if (surfaceUrl != null) map['SurfaceUrl'] = surfaceUrl;
    if (mdFileUrl != null) map['MdFileUrl'] = mdFileUrl;
    map['CarotidScanType'] = carotidScanType.index;
    map['CarotidScanDirection'] = carotidScanDirection.index;
    return map;
  }
}

class Ultr3DParameter {
  Ultr3DParameter(this.recordID, this.resourceInfos);
  String recordID;
  List<Ultra3DResourceInfo> resourceInfos;

  String toJson() {
    return json.encode({
      "RecordID": recordID,
      "ResourceInfos": resourceInfos.map((e) => e.toJson()).toList(),
    });
  }
}
