import 'dart:ui';
import 'package:fis_common/index.dart';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/measure_page_test.dart';
import 'package:fis_measure/process/layout/configuration.dart';
import 'package:fis_measure/view/measure/measure_view.dart';
import 'package:fis_vid/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:fis_i18n/i18n.dart';

import 'third_vid_test.dart';

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  logger.setMinimumLevel(LogLevel.UserOperate);
  await logger.init();

  await _initI18n();
  await LayoutConfiguration.ins.loadData();
  // LayoutConfiguration.ins.getRect("")
  final rpc = JsonRpcProxy();
  if (FPlatform.isMacOS || FPlatform.isWindows) {
    VidDataHostEnv.setPlatformGetter(() => rpc.platform);
  }
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      debugShowCheckedModeBanner: false,
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: const MyHomePage(title: 'Flutter Demo Home Page'),
      // home: ImageDemoPage(),
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({Key? key, required this.title}) : super(key: key);
  final String title;

  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  void _incrementCounter() {}

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.title),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            ElevatedButton(
              onPressed: () {
                ThirdPartVidTest().run();
              },
              child: const Text("Test 3rd vid"),
            ),
            const SizedBox(height: 10),
            ElevatedButton(
              onPressed: () {
                Navigator.of(context).push(
                  MaterialPageRoute(
                    builder: (context) => const MeasureTestPage(),
                  ),
                );

                // MeasureController("").load();
                // const url =
                //     "http://192.168.6.117:9001/Flyinsono-BJ-1300984704.VCS.AP-BeiJing/default.VID";
                // // final isMobile = Platform.isAndroid || Platform.isIOS;
                // Navigator.of(context).push(
                //   MaterialPageRoute(
                //     builder: (_) => VidPlayerPage(url),
                //     // !kIsWeb ? VidPlayerMobilePage(url) : VidPlayerPage(url),
                //   ),
                // );
              },
              child: const Text("Player"),
            ),
            const SizedBox(
              height: 15,
            ),
            ElevatedButton(
              onPressed: () {
                Navigator.of(context).push(
                  MaterialPageRoute(
                    builder: (context) => const MeasureMainPage(
                        '50BB5ED499304A8C8BD77B39CBCF596E',
                        'RecordInfoDO_20220811025200rOow04',
                        'PatientInfoDO_202208110311443tup96',
                        'RemedicalDO_20220811025202y3u0Rc'),
                  ),
                );
              },
              child: const Text("Measure image"),
            ),
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: _incrementCounter,
        tooltip: 'Increment',
        child: const Icon(Icons.add),
      ), // This trailing comma makes auto-formatting nicer for build methods.
    );
  }
}

Future<void> _initI18n() async {
  final curLocale = window.locale;

  var bookOptions = [
    FTrKeybookOption.ChineseOption,
    FTrKeybookOption.EnglishOption,
  ];

  await FI18n.init((locale) => Get.updateLocale(locale), bookOptions);
  await FI18n.load(curLocale);

  Get.put(FI18n.ins);
}
