import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/calculators/output.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:vid/us/vid_us_unit.dart';

import '../physical_coordinates/convex_tissue.dart';
import '../primitives/location.dart';
import 'calculator.dart';

class TissueDepthCal extends Calculator<Location, double> {
  TissueDepthCal(Location ref) : super(ref);

  @override
  void calculate() {
    if (ref.feature == null) return;

    // TODO:xxx
    final viewport = ref.feature!.hostVisualArea!.viewport!;
    final point = ref.feature!.innerPoints.first;
    final physicalPoint = viewport.physical!.convert(point);
    final value = physicalPoint.y * viewport.region.height;

    final outputItem = createOutput(MeasureTypes.Depth, value, VidUsUnit.cm);
    final description =
        "${ref.description}: ${roundDouble(value)}${VidUsUnit.cm.name}";
    outputItem.updateDescription(description: description);
    output = outputItem;

    updateFloatValue(value);
  }
}

class TissueConvexDepthCal extends Calculator<Location, double> {
  TissueConvexDepthCal(Location ref) : super(ref);

  @override
  void calculate() {
    if (ref.feature == null) return;

    // TODO:xxx
    final layout = ref.feature!.hostVisualArea!.displayRegion;
    final viewport = ref.feature!.hostVisualArea!.viewport!;
    final physical = viewport.physical! as ConvexTissuePhysicalCoordinate;
    final point = ref.feature!.point.clone();

    point.addOffset(-layout.left, -layout.top);

    final physicalPoint = viewport.convert(point);
    final physicalZeroPoint = DPoint(viewport.region.width / 2, physical.zeroY);
    final distance = (physicalPoint - physicalZeroPoint).length;
    final depth = distance - physical.zeroRadius;

    final valueDesc =
        depth < 0 ? '' : '${roundDouble(depth)}${VidUsUnit.cm.name}';
    final description = "${ref.description}: $valueDesc";

    final double value = depth < 0 ? 0 : depth;
    final outputItem = createOutput(MeasureTypes.Depth, value, VidUsUnit.cm);

    outputItem.updateDescription(description: description);
    output = outputItem;
    updateFloatValue(value);
  }
}
