// ignore_for_file:  constant_identifier_names

import 'dart:math' as math;

import 'package:fis_measure/interfaces/date_types/point.dart';

class GeneralFormulas {
  /// Volume:1/6 x π
  static const double VolumeCofficient = math.pi / 6.0;

  /// 带比率系数算体积
  ///
  /// [coefficient] 比率系数
  static double volumeWithCoefficient(
    double d1,
    double d2,
    double d3,
    double coefficient,
  ) {
    double volume = 0;
    if (!doubleAlmostEquals(d1, 0) &&
        !doubleAlmostEquals(d2, 0) &&
        !doubleAlmostEquals(d3, 0)) {
      volume = d1 * d2 * d3 * coefficient;
    }
    return volume;
  }

  static bool doubleAlmostEquals(double num1, double num2,
      [double precision = 0.000001]) {
    if (num1.isNaN && num2.isNaN) return true;
    return (num1 - num2).abs() <= precision;
  }

  /// 点到线的距离
  ///
  /// [pt1] 线端点1
  ///
  /// [pt2] 线端点2
  ///
  /// [pt] 点
  static double distance2Line(DPoint pt1, DPoint pt2, DPoint pt) {
    double dis = 0;
    if (pt1.x == pt2.x) {
      dis = (pt.x - pt1.x).abs();
      return dis;
    }
    var lineK = (pt2.y - pt1.y) / (pt2.x - pt1.x);
    var lineC = (pt2.x * pt1.y - pt1.x * pt2.y) / (pt2.x - pt1.x);
    dis = (lineK * pt.x - pt.y + lineC).abs() / (math.sqrt(lineK * lineK + 1));
    return dis;
  }
}
