import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/calculators/output.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:fis_measure/process/primitives/ellipse.dart';
import 'package:flutter/cupertino.dart';
import 'package:vid/us/vid_us_unit.dart';

import '../primitives/trace.dart';
import 'calculator.dart';

class TracePerimeterCal extends Calculator<Trace, double> {
  TracePerimeterCal(Trace ref) : super(ref);

  @override
  void calculate() {
    if (ref.feature == null) return;

    // TODO:xxx
    final feature = ref.feature!;
    final viewport = feature.hostVisualArea!.viewport!;
    final points = feature.innerPoints.map((e) => viewport.convert(e)).toList();

    double threshold = 0;
    // if (ref.Threshold.HasValue)
    // {
    //     threshold = ref.Threshold.Value;
    // }
    double value = calcPerimeter(points, ref.feature!.isClosed, threshold);
    final outputItem =
        createOutput(MeasureTypes.Perimeter, value, VidUsUnit.cm);
    final description =
        "${ref.description}: ${roundDouble(value)}${VidUsUnit.cm.name}";
    outputItem.updateDescription(description: description);
    output = outputItem;
    updateFloatValue(value);
  }

  static double calcPerimeter(
    List<DPoint> points,
    bool isClosed,
    double threshold,
  ) {
    if (points.length < 2) {
      return 0;
    }

    double sum = 0;
    for (int i = 1, j = 0; j < points.length && i < points.length; i++) {
      var length = (points[i] - points[j]).length;
      if (length.abs() > threshold) {
        sum += length;
        j = i;
      }
    }
    if (isClosed) {
      sum += (points[points.length - 1] - points[0]).length;
    }
    return sum;
  }
}

class AreaPerimeterEllipseCal extends Calculator<Ellipse, double> {
  AreaPerimeterEllipseCal(Ellipse ref) : super(ref);

  @override
  void calculate() {
    if (ref.feature == null) return;

    final feature = ref.feature!;
    final viewport = feature.hostVisualArea!.viewport!;

    for (var output in ref.meta.outputs) {
      if (output.name == MeasureTypes.Perimeter) {
        var value = ref.feature!.getCircumference(viewport.convertBoundary);
        feature.updateFloatValue(output, value, output.unit);
      } else if (output.name == MeasureTypes.Area) {
        var value = ref.feature!.getArea(viewport.convertBoundary);
        feature.updateFloatValue(output, value, output.unit);
      }
    }
  }
}
