import 'package:fis_measure/interfaces/process/calculators/output.dart';
import 'package:fis_measure/process/calcuators/formulas/general.dart';
import 'package:fis_measure/process/primitives/combos/lwh_straightline.dart';
import 'package:fis_measure/process/primitives/ellipse.dart';
import 'package:fis_measure/process/primitives/straightline.dart';

import 'calculator.dart';

class VolumeThreeDistanceCal extends Calculator<LWHStraightLine, double> {
  VolumeThreeDistanceCal(LWHStraightLine ref) : super(ref);

  @override
  void calculate() {
    if (ref.feature == null) return;

    final outputMeta = ref.meta.outputs.first;
    final outputItem = createOutput(outputMeta.name, 0, outputMeta.unit);
    final l = _pickChildOutput(ref.l);
    final w = _pickChildOutput(ref.w);
    final h = _pickChildOutput(ref.h);

    String description;
    final feature = ref.feature!;
    final viewport = feature.hostVisualArea!.viewport!;
    if (l != null && w != null && h != null) {
      final value = GeneralFormulas.volumeWithCoefficient(
        l.value,
        w.value,
        h.value,
        GeneralFormulas.VolumeCofficient,
      );
      updateFloatValue(value);
      outputItem.value = value;
      description = "${ref.description}  ${roundDouble(value)}cm³";
    } else {
      description = ref.description;
    }

    outputItem.updateDescription(description: description);
    output = outputItem;
  }

  OutputItem? _pickChildOutput(StraightLine item) {
    if (item.calculator == null) return null;

    if (item.calculator!.outputs.isEmpty) {
      return item.calculator!.output;
    } else {
      return item.calculator!.outputs.first;
    }
  }
}

class VolumeEllipseCal extends Calculator<Ellipse, double> {
  VolumeEllipseCal(Ellipse ref) : super(ref);

  @override
  void calculate() {
    // TODO: implement calculate
  }
}
