import 'dart:core';

import 'package:fis_common/event/event_type.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/enums/annotation.dart';
import 'package:fis_measure/view/measure/tool_chest_title.dart';

class ChangeImageInfo {
  /// 图片地址
  final String? imageUrl;

  /// 图像编码
  final String? remedicalCode;
  ChangeImageInfo(
    this.imageUrl,
    this.remedicalCode,
  );
}

class MeasureModeSelection {
  /// 应用名称：读取Vid获取应用
  final String? applicationName;

  /// 分类名称：读取Vid获取应用分类名称
  final String? categoryName;

  /// 测量模式集合：读取Vid获取应用的模式集合，返回对应测量数据
  final List<String>? measureModes;
  MeasureModeSelection(
    this.applicationName,
    this.categoryName,
    this.measureModes,
  );
}

class MeasureModeSubmit {
  /// 版本号
  final String? version;

  /// 应用名称，读取Vid获取应用
  final String? applicationName;

  /// 分类名称：读取Vid获取应用分类名称
  final String? categoryName;

  ///  测量模式
  final UserDefinedMeasureModeDTO? workingMode;
  MeasureModeSubmit(
    this.version,
    this.applicationName,
    this.categoryName,
    this.workingMode,
  );
}

abstract class IMeasureHandler {
  /// 测量切换图片事件
  late FEventHandler<ChangeImageInfo> onImageChanged;

  /// 测量方式选择
  late FEventHandler<MeasureModeSelection> onMeasureModeSelectionChanged;

  /// 测量方式提交
  late FEventHandler<MeasureModeSubmit> onMeasureModeSubmitChanged;

  /// 测量还是注释tab
  late FEventHandler<TagEnum> onChangedTab;

  /// 注释箭头还是标签
  late FEventHandler<AnnotationType> onChangedAnnotationType;

  /// 图片加载loadding
  late FEventHandler<bool> onChangeImageLoaded;

  late FEventHandler<bool> onChangeFullScreenState;

  /// 切换图像传递数据
  ChangeImageInfo get imageChanged;
  set imageChanged(ChangeImageInfo value);

  /// 切换图像测量方式变更数据
  MeasureModeSelection get measureModeChanged;
  set measureModeChanged(MeasureModeSelection value);

  /// 测量方式变更提交
  MeasureModeSubmit get measureModeSubmitChanged;
  set measureModeSubmitChanged(MeasureModeSubmit value);

  /// 切换Tab
  TagEnum get changedTab;
  set changedTab(TagEnum value);

  /// 切换注释
  AnnotationType get changedAnnotationType;
  set changedAnnotationType(AnnotationType value);

  /// 切换图片
  bool get changeImageLoaded;
  set changeImageLoaded(bool value);

  /// 全屏状态
  bool get fullScreenState;
  set fullScreenState(bool value);
}

class MeasureHandler implements IMeasureHandler {
  ChangeImageInfo _imageChanged = ChangeImageInfo(null, null);
  MeasureModeSelection _measureModeChanged =
      MeasureModeSelection(null, null, null);

  MeasureModeSubmit _measureModeSubmitChanged =
      MeasureModeSubmit(null, null, null, null);

  TagEnum _changedTab = TagEnum.MeasureTool;

  bool _changeImageLoaded = true;

  bool _fullScreenState = false;

  AnnotationType _changedAnnotationType = AnnotationType.label;
  @override
  var onImageChanged = FEventHandler<ChangeImageInfo>();

  @override
  var onMeasureModeSelectionChanged = FEventHandler<MeasureModeSelection>();

  @override
  var onMeasureModeSubmitChanged = FEventHandler<MeasureModeSubmit>();
  @override
  var onChangedTab = FEventHandler<TagEnum>();

  @override
  var onChangedAnnotationType = FEventHandler<AnnotationType>();
  @override
  FEventHandler<bool> onChangeImageLoaded = FEventHandler<bool>();

  @override
  FEventHandler<bool> onChangeFullScreenState = FEventHandler<bool>();

  @override
  ChangeImageInfo get imageChanged => _imageChanged;

  @override
  MeasureModeSelection get measureModeChanged => _measureModeChanged;

  @override
  MeasureModeSubmit get measureModeSubmitChanged => _measureModeSubmitChanged;

  @override
  TagEnum get changedTab => _changedTab;

  @override
  AnnotationType get changedAnnotationType => _changedAnnotationType;

  @override
  bool get changeImageLoaded => _changeImageLoaded;

  @override
  bool get fullScreenState => _fullScreenState;
  @override
  set fullScreenState(bool value) {
    if (value != _fullScreenState) {
      _fullScreenState = value;
      _onChangeFullScreenState();
    }
  }

  @override
  set changeImageLoaded(bool value) {
    if (value != _changeImageLoaded) {
      _changeImageLoaded = value;
      _onChangeImageLoaded();
    }
  }

  @override
  set changedTab(TagEnum value) {
    if (value != _changedTab) {
      _changedTab = value;
      _onChangedTab();
    }
  }

  @override
  set changedAnnotationType(AnnotationType value) {
    if (value != _changedAnnotationType) {
      _changedAnnotationType = value;
      _onChangedAnnotation();
    }
  }

  @override
  set imageChanged(ChangeImageInfo value) {
    if (value != _imageChanged) {
      _imageChanged = value;
      _onImageChanged();
    }
  }

  @override
  set measureModeChanged(MeasureModeSelection value) {
    if (value != _measureModeChanged) {
      _measureModeChanged = value;
      _onMeasureModeSelectionChanged();
    }
  }

  @override
  set measureModeSubmitChanged(MeasureModeSubmit value) {
    if (value != _measureModeSubmitChanged) {
      _measureModeSubmitChanged = value;
      _onMeasureModeSubmitChanged();
    }
  }

  void _onImageChanged() {
    onImageChanged.emit(this, imageChanged);
  }

  void _onMeasureModeSelectionChanged() {
    onMeasureModeSelectionChanged.emit(this, measureModeChanged);
  }

  void _onMeasureModeSubmitChanged() {
    onMeasureModeSubmitChanged.emit(this, measureModeSubmitChanged);
  }

  void _onChangedTab() {
    onChangedTab.emit(this, changedTab);
  }

  void _onChangeImageLoaded() {
    onChangeImageLoaded.emit(this, changeImageLoaded);
  }

  void _onChangeFullScreenState() {
    onChangeFullScreenState.emit(this, fullScreenState);
  }

  void _onChangedAnnotation() {
    onChangedAnnotationType.emit(this, changedAnnotationType);
  }
}
