import 'package:fis_measure/index.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/interfaces/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class Measure3DViewSwitcher extends StatefulWidget implements FWidget {
  const Measure3DViewSwitcher(this.measure3DViewController, {Key? key})
      : super(key: key);
  final Measure3DViewController measure3DViewController;
  @override
  State<StatefulWidget> createState() => _Measure3DViewSwitcherState();
}

class _Measure3DViewSwitcherState extends State<Measure3DViewSwitcher> {
  bool enable3DView = false;
  bool ifShow3DSwitcher = false;
  final playerController = Get.find<IPlayerController>() as VidPlayerController;
  @override
  void initState() {
    super.initState();
    enable3DView = widget.measure3DViewController.curMeasureMode ==
        MeasureMode.carotid3DMode;
    widget.measure3DViewController.updatePlayerMode.addListener(_onModeChanged);
  }

  /// 模式改变触发更新
  void _onModeChanged(Object s, MeasureMode mode) {
    switch (mode) {
      case MeasureMode.vidMode:
        setState(() {
          enable3DView = false;
        });
        break;
      case MeasureMode.carotid2DMode:
        setState(() {
          ifShow3DSwitcher = true;
        });
        break;
      case MeasureMode.carotid3DMode:
        break;
    }
  }

  @override
  void dispose() {
    widget.measure3DViewController.updatePlayerMode
        .removeListener(_onModeChanged);
    super.dispose();
  }

  @override
  FWidget build(BuildContext context) {
    return FContainer(
      width: 300,
      padding: const EdgeInsets.all(10),
      child: FColumn(
          mainAxisSize: MainAxisSize.min,
          children: [ifShow3DSwitcher ? modeButton() : switcher()]),
    );
  }

  FWidget switcher() {
    return FRow(
        mainAxisAlignment: MainAxisAlignment.center,
        mainAxisSize: MainAxisSize.min,
        children: [
          const FSizedBox(
              width: 90,

              ///TODO: [Gavin] i18n
              child: FText("启用 3D :", style: TextStyle(color: Colors.white))),
          FSwitch(
            value: enable3DView,
            onChanged: (value) {
              playerController.pause();
              if (value) {
                widget.measure3DViewController.changeModeTo3DMode();
              } else {
                widget.measure3DViewController.backToVidMode();
                playerController.resetCurrentFrame();
              }
              setState(() {
                enable3DView = value;
              });
            },
          ),
        ]);
  }

  FWidget modeButton() {
    return FRow(
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
      children: [
        FElevatedButton(
          onPressed: () {
            widget.measure3DViewController.changeModeTo3DMode();
            setState(() {
              ifShow3DSwitcher = false;
              enable3DView = true;
            });
          },
          child: const FText(
            ///TODO: [Gavin] i18n
            "回到 3D",
            style: TextStyle(color: Colors.white),
          ),
        ),
        FElevatedButton(
          onPressed: () {
            widget.measure3DViewController.backToVidMode();
            playerController.resetCurrentFrame();
            setState(() {
              ifShow3DSwitcher = false;
              enable3DView = false;
            });
          },
          child: const FText(
            ///TODO: [Gavin] i18n
            "回到播放器",
            style: TextStyle(color: Colors.white),
          ),
        ),
      ],
    );
  }
}
