import 'package:fis_measure/interfaces/process/annotations/annotation.dart';
import 'package:fis_measure/interfaces/process/items/item_feature.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class MeasureRecordsCanvasPanel extends StatefulWidget {
  const MeasureRecordsCanvasPanel({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _RecordsCanvasPanelState();
}

class _RecordsCanvasPanelState extends State<MeasureRecordsCanvasPanel> {
  late final application = Get.find<IApplication>();
  late List<IMeasureItemFeature> measureFeatures = [];
  late List<IAnnotationItemFeature> annotationFeatures = [];

  bool get hasNoData => measureFeatures.isEmpty && annotationFeatures.isEmpty;

  @override
  void initState() {
    _addListenrs();
    super.initState();
  }

  @override
  void dispose() {
    _removeListenrs();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (hasNoData) {
      return const SizedBox();
    }

    return LayoutBuilder(builder: (context, constraints) {
      return SizedBox(
        width: constraints.maxWidth,
        height: constraints.maxHeight,
        child: RepaintBoundary(
          child: CustomPaint(
            painter: _PanelPainter(measureFeatures, annotationFeatures),
          ),
        ),
      );
    });
  }

  void _updateFeatures() {
    measureFeatures.clear();
    for (var item in application.measureItems) {
      measureFeatures.addAll(item.measuredFeatures);
    }

    annotationFeatures.clear();
    for (var item in application.annotationItems) {
      annotationFeatures.addAll(item.features);
    }
  }

  void _onMeasureRerenderReady(Object sender, void e) {
    if (mounted) {
      setState(() {
        _updateFeatures();
      });
    }
  }

  void _addListenrs() {
    application.updateRenderReady.addListener(_onMeasureRerenderReady);
  }

  void _removeListenrs() {
    application.updateRenderReady.removeListener(_onMeasureRerenderReady);
  }
}

class _PanelPainter extends CustomPainter {
  final List<IMeasureItemFeature> measureFeatures;
  final List<IAnnotationItemFeature> annotationFeatures;
  _PanelPainter(this.measureFeatures, this.annotationFeatures);

  @override
  void paint(Canvas canvas, Size size) {
    for (final feature in measureFeatures) {
      feature.paint(canvas, size);
    }
    for (final feature in annotationFeatures) {
      feature.paint(canvas, size);
    }
  }

  @override
  bool shouldRepaint(covariant _PanelPainter oldDelegate) {
    return oldDelegate.hashCode != hashCode;
  }
}
