import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/view/cursor.dart';
import 'package:fis_measure/view/gesture/positioned_cursor.dart';
import 'package:fis_theme/theme.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

part './measure_configuration_style_components.dart';

/// 下拉选择的类
class MeasureSelectModel {
  MeasureSelectModel({required this.name, required this.code});
  final String name;
  final String code;
}

MeasureCursorType getMeasureSystemSettingCursorType(
  CursorTypeEnum cursorType,
) {
  switch (cursorType) {
    case CursorTypeEnum.CursorType1Icon:
      return MeasureCursorType.cursor01;
    case CursorTypeEnum.CursorType2Icon:
      return MeasureCursorType.cursor02;
    case CursorTypeEnum.CursorType3Icon:
      return MeasureCursorType.cursor03;
    case CursorTypeEnum.CursorType4Icon:
      return MeasureCursorType.cursor04;
    case CursorTypeEnum.CursorType5Icon:
      return MeasureCursorType.cursor05;
    default:
      return MeasureCursorType.cursor01;
  }
}

class PatternDialog extends StatelessWidget {
  PatternDialog({
    Key? key,
  }) : super(key: key);
  MeasureSystemSettingDTO measureSystemSetting = MeasureSystemSettingDTO();

  /// 测量数据
  final measureData = Get.find<MeasureDataController>();
  final mouseState = Get.put<IMouseState>(MouseState());

  @override
  FWidget build(BuildContext context) {
    return FSimpleDialog(
      title: FText(
        i18nBook.measure.style.t,
        style: const TextStyle(
          color: Colors.white,
          fontSize: 18,
        ),
      ),
      isDefault: true,
      onOk: () {
        _saveMeasureSystemSettingAsync();
        Get.back();
      },
      onCancel: () {
        Get.back();
      },
      children: [
        FContainer(
            width: 550,
            height: 500,
            padding: const EdgeInsets.symmetric(
              vertical: 15,
              horizontal: 15,
            ),
            margin: const EdgeInsets.symmetric(
              horizontal: 15,
            ),
            child: FFutureBuilder(
              builder: (context, snapshot) {
                if (snapshot.connectionState == ConnectionState.done) {
                  if (snapshot.hasError) {
                    return FText("Error: ${snapshot.error}");
                  } else {
                    return PatternBody(
                      measureSystemSetting: measureData.measureSystemSetting,
                    );
                  }
                } else {
                  return const FCenter(
                    child: FSizedBox(
                      width: 50,
                      height: 50,
                      child: FCircularProgressIndicator(),
                    ),
                  );
                }
              },
              future: _getMeasureSystemSetting(),
            )),
      ],
    );
  }

  Future<void> _saveMeasureSystemSettingAsync() async {
    await measureData
        .saveMeasureSystemSettingAsync(measureData.measureSystemSetting);
    mouseState.cursorType = getMeasureSystemSettingCursorType(
        measureData.measureSystemSetting.cursorType);
    mouseState.cursorSize =
        measureData.measureSystemSetting.cursorSize as double;
  }

  Future<void> _getMeasureSystemSetting() async {
    final result = await measureData.getMeasureSystemSettingAsync();
    measureData.measureSystemSetting = result as MeasureSystemSettingDTO;
  }
}

class MeasureTextSize {
  MeasureTextSize(this.textSize);
  final int textSize;

  @override
  String toString() {
    return '$textSize×$textSize';
  }
}

class PatternBody extends FStatefulWidget {
  const PatternBody({
    Key? key,
    required this.measureSystemSetting,
  }) : super(key: key);
  final MeasureSystemSettingDTO measureSystemSetting;
  @override
  FState<PatternBody> createState() => _PatternBodyState();
}

class _PatternBodyState extends FState<PatternBody> {
  /// 测量项文本大小
  static final List<int> C_MEASURE_TEXT_SIZE = [
    10,
    16,
    32,
  ];
  final mouseState = Get.put<IMouseState>(MouseState());

  /// 测量项标识枚举
  static final List<MeasureCursorType> C_MEASURE_CURSORTYPE = [
    MeasureCursorType.cursor01,
    MeasureCursorType.cursor02,
    MeasureCursorType.cursor03,
    MeasureCursorType.cursor04,
    MeasureCursorType.cursor05,
  ];

  /// 测量项下拉文本大小
  static final List<MeasureSelectModel> C_MEASURE_DROP_TEXT_SIZE = [
    MeasureSelectModel(
      name: '10',
      code: '10',
    ),
    MeasureSelectModel(
      name: '18',
      code: '18',
    ),
    MeasureSelectModel(
      name: '20',
      code: '20',
    ),
    MeasureSelectModel(
      name: '32',
      code: '32',
    ),
  ];

  /// 测量数据
  late final measureData = Get.find<MeasureDataController>();

  @override
  void initState() {
    super.initState();
  }

  CursorTypeEnum getMeasureSystemSettingCursorTypeIcon(
    MeasureCursorType cursorType,
  ) {
    switch (cursorType) {
      case MeasureCursorType.cursor01:
        return CursorTypeEnum.CursorType1Icon;

      case MeasureCursorType.cursor02:
        return CursorTypeEnum.CursorType2Icon;

      case MeasureCursorType.cursor03:
        return CursorTypeEnum.CursorType3Icon;

      case MeasureCursorType.cursor04:
        return CursorTypeEnum.CursorType4Icon;

      case MeasureCursorType.cursor05:
        return CursorTypeEnum.CursorType5Icon;

      default:
        return CursorTypeEnum.CursorType1Icon;
    }
  }

  @override
  FWidget build(BuildContext context) {
    return FListView(
      shrinkWrap: true,
      children: [
        _PatternBodyItem(
          patternBodyItemTitle: i18nBook.measure.ruler.t,
          patternBodyItem: FColumn(
            children: [
              _PatternItem(
                title: i18nBook.measure.rulerType.t,
                item: _PatternItemRadio(
                    tabList: C_MEASURE_CURSORTYPE,
                    value: getMeasureSystemSettingCursorType(
                      widget.measureSystemSetting.cursorType,
                    ),
                    isDefaultType: false,
                    onChangeState: (value) {
                      setState(
                        () {
                          measureData.measureSystemSetting.cursorType =
                              getMeasureSystemSettingCursorTypeIcon(value);
                        },
                      );
                    }),
              ),
              _PatternItem(
                title: i18nBook.measure.rulerSize.t,
                item: _PatternItemRadio(
                  tabList: C_MEASURE_TEXT_SIZE,
                  value: widget.measureSystemSetting.cursorSize,
                  onChangeState: (value) {
                    setState(() {
                      measureData.measureSystemSetting.cursorSize = value;
                    });
                  },
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.measureRulerSize.t,
                item: _PatternItemRadio(
                  tabList: C_MEASURE_TEXT_SIZE,
                  value: widget.measureSystemSetting.shapeCursorSize,
                  onChangeState: (value) {
                    setState(() {
                      measureData.measureSystemSetting.shapeCursorSize = value;
                    });
                  },
                ),
              ),
            ],
          ),
        ),
        _PatternBodyItem(
          patternBodyItemTitle: i18nBook.measure.resultWindowConfig.t,
          patternBodyItem: FColumn(
            children: [
              _PatternItem(
                title: i18nBook.measure.showResultWindow.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      measureData.measureSystemSetting.showResultWindow = value;
                    });
                  },
                  value: widget.measureSystemSetting.showResultWindow,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.fontSize.t,
                item: _PatternItemSelect(
                  textSize: C_MEASURE_DROP_TEXT_SIZE,
                  value: widget.measureSystemSetting.fontSize.toString(),
                  onChanged: (value) {
                    setState(() {
                      measureData.measureSystemSetting.fontSize = value;
                    });
                  },
                ),
              ),
            ],
          ),
        ),
        _PatternBodyItem(
          patternBodyItemTitle: i18nBook.measure.display.t,
          isEnd: true,
          patternBodyItem: FColumn(
            children: [
              _PatternItem(
                title: i18nBook.measure.showRulerLine.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      measureData.measureSystemSetting.showCursorLine = value;
                    });
                  },
                  value: widget.measureSystemSetting.showCursorLine,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.keepMeasureOnImage.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      measureData.measureSystemSetting.holdMeasureLine = value;
                    });
                  },
                  value: widget.measureSystemSetting.holdMeasureLine,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.showDepthMeasureLine.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.measureSystemSetting.showDepthGuideline = value;
                    });
                  },
                  value: widget.measureSystemSetting.showDepthGuideline,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.showSimpleAnnotation.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.measureSystemSetting.showBriefAnnotation = value;
                    });
                  },
                  value: widget.measureSystemSetting.showBriefAnnotation,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.autoAdjustCursor.t,
                item: _PatternItemSelect(
                  textSize: C_MEASURE_DROP_TEXT_SIZE,
                  value:
                      widget.measureSystemSetting.minCursorDistance.toString(),
                  onChanged: (value) {
                    setState(() {
                      widget.measureSystemSetting.minCursorDistance = value;
                    });
                  },
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.autoCaptureRange.t,
                item: _PatternItemSelect(
                  textSize: C_MEASURE_DROP_TEXT_SIZE,
                  value:
                      widget.measureSystemSetting.autoSnapDistance.toString(),
                  onChanged: (value) {
                    setState(() {
                      widget.measureSystemSetting.autoSnapDistance = value;
                    });
                  },
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.annotationFontSize.t,
                item: _PatternItemSelect(
                  textSize: C_MEASURE_DROP_TEXT_SIZE,
                  value:
                      widget.measureSystemSetting.annotationFontSize.toString(),
                  onChanged: (value) {
                    setState(() {
                      widget.measureSystemSetting.annotationFontSize = value;
                    });
                  },
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.showSubItemResultInMeasureWorkSheet.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.measureSystemSetting.showProtocolInWorkSheet =
                          value;
                    });
                  },
                  value: widget.measureSystemSetting.showProtocolInWorkSheet,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.showMeasureAnnotation.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.measureSystemSetting.showAnnotation = value;
                    });
                  },
                  value: widget.measureSystemSetting.showAnnotation,
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }
}
