part of 'measure_configuration_style.dart';

class _PatternBodyItem extends FStatelessWidget {
  final String patternBodyItemTitle;
  final FWidget patternBodyItem;
  final bool? isEnd;
  const _PatternBodyItem({
    Key? key,
    required this.patternBodyItemTitle,
    required this.patternBodyItem,
    this.isEnd = false,
  }) : super(key: key);
  @override
  FWidget build(BuildContext context) {
    return FColumn(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        FText(
          patternBodyItemTitle,
          style: const TextStyle(
            fontWeight: FontWeight.bold,
          ),
        ),
        const FSizedBox(
          height: 8,
        ),
        patternBodyItem,
        if (!isEnd!)
          FContainer(
            decoration: const BoxDecoration(
              border: Border(
                bottom: BorderSide(
                  color: Color.fromRGBO(235, 235, 235, 1),
                  width: 1,
                ),
              ),
            ),
          ),
        const FSizedBox(
          height: 16,
        ),
      ],
    );
  }
}

class _PatternItem extends FStatelessWidget {
  final String title;
  final FWidget item;
  const _PatternItem({required this.title, required this.item, Key? key})
      : super(key: key);
  @override
  FWidget build(BuildContext context) {
    return FContainer(
      margin: const EdgeInsets.only(bottom: 8),
      child: FRow(
        children: [
          FContainer(
            width: 200,
            child: FText(title),
          ),
          FExpanded(
            child: FContainer(
              alignment: Alignment.centerLeft,
              child: item,
            ),
          ),
          const FSizedBox(
            width: 20,
          ),
        ],
      ),
    );
  }
}

class _PatternItemRadio extends FStatelessWidget {
  final List<dynamic> tabList;
  final bool? isDefaultType;
  final dynamic value;
  final ValueCallback onChangeState;
  const _PatternItemRadio({
    Key? key,
    required this.tabList,
    required this.value,
    required this.onChangeState,
    this.isDefaultType = true,
  }) : super(key: key);

  @override
  FWidget build(BuildContext context) {
    return FContainer(
      margin: const EdgeInsets.only(left: 10),
      child: FWrap(
        spacing: 8.0, // 主轴(水平)方向间距
        runSpacing: 8.0, // 纵轴（垂直）方向间距
        direction: Axis.horizontal,
        crossAlignment: WrapCrossAlignment.center,
        alignment: WrapAlignment.start,
        children: List<FWidget>.generate(
          tabList.length,
          (index) {
            final FWidget child;
            if (isDefaultType!) {
              child = FText(
                MeasureTextSize(tabList[index]).toString(),
                style: tabList[index] != value
                    ? const TextStyle(
                        color: Colors.black,
                      )
                    : const TextStyle(
                        color: Colors.white,
                      ),
              );
            } else {
              child = MeasureCursor(
                size: 10,
                type: tabList[index],
                color: tabList[index] != value ? Colors.black : Colors.white,
              );
            }
            return FElevatedButton(
              onPressed: () {
                if (tabList[index] != value) onChangeState.call(tabList[index]);
              },
              style: ButtonStyle(
                backgroundColor: tabList[index] != value
                    ? MaterialStateProperty.all(
                        Colors.white,
                      )
                    : MaterialStateProperty.all(
                        FTheme.ins.colorScheme.primary,
                      ),
              ),
              child: FContainer(
                width: isDefaultType! ? 50 : 20,
                height: 20,
                child: FCenter(
                  child: child,
                ),
              ),
            );
          },
        ),
      ),
    );
  }
}

class _PatternItemSwitch extends FStatelessWidget {
  const _PatternItemSwitch({
    Key? key,
    required this.onChanged,
    required this.value,
  }) : super(key: key);
  final ValueCallback onChanged;
  final bool value;

  @override
  FWidget build(BuildContext context) {
    return FSwitch(
      inactiveThumbColor: FTheme.ins.colorScheme.primary,
      inactiveTrackColor: Colors.grey.withAlpha(88),
      activeColor: Colors.white,
      splashRadius: 0,
      activeTrackColor: FTheme.ins.colorScheme.primary,
      onChanged: (value) => onChanged.call(value),
      value: value,
    );
  }
}

class _PatternItemSelect extends StatelessWidget implements FWidget {
  const _PatternItemSelect({
    required this.onChanged,
    required this.value,
    required this.textSize,
  });
  final ValueCallback onChanged;
  final String value;
  final List<MeasureSelectModel> textSize;
  @override
  FWidget build(BuildContext context) {
    return FContainer(
      margin: const EdgeInsets.symmetric(
        vertical: 4,
        horizontal: 10,
      ),
      child: FSelect<MeasureSelectModel, String>(
        source: textSize,
        hintText: i18nBook.measure.selectAnnotationFontSize.t,
        value: value,
        clearable: false,
        height: 36,
        width: 400,
        optionLabelExtractor: (data) => data.name,
        optionValueExtractor: (data) => data.code,
        onSelectChanged: (value, index) => onChanged.call(value),
      ),
    );
  }
}
