// ignore: must_be_immutable
import 'dart:convert';

import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/view/measure/measure_config/measure_configuation_page.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class SelectModulePage extends FStatefulWidget {
  final List<ItemMetaGroup> itemMetaListGroup;

  /// 选中的测量项
  List<String> chooseMeasureList = [];
  SelectModulePage({
    required this.itemMetaListGroup,
    required this.chooseMeasureList,
  });
  @override
  FState<SelectModulePage> createState() => _SelectModulePageState();
}

class _SelectModulePageState extends FState<SelectModulePage> {
  /// 数据
  final measureData = Get.find<MeasureDataController>();

  /// 根据语言包翻译
  ///
  /// [code] 翻译编码
  String getValues(String code) {
    final value = jsonDecode(measureData.measureLanguage)["Language"][code];
    return value ?? code;
  }

  /// 新增测量项目
  ///
  /// [chooseMeasure] 选中的测量项
  void addChooseMeasure(String chooseMeasure) {
    widget.chooseMeasureList.add(
      chooseMeasure,
    );
    setState(() {});
  }

  /// 移除测量项目
  ///
  /// [chooseMeasure] 移除的测量项
  void removeChooseMeasure(String chooseMeasure) {
    widget.chooseMeasureList.remove(
      chooseMeasure,
    );
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  FWidget build(BuildContext context) {
    return FListView(
      shrinkWrap: true,
      children: widget.itemMetaListGroup
          .map(
            (e) => FColumn(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: _buildMeasureConfiguration(e),
            ),
          )
          .toList(),
    );
  }

  List<FWidget> _buildMeasureConfiguration(ItemMetaGroup e) {
    return [
      FInkWell(
        onTap: () {
          e.isExpend = !e.isExpend!;
          setState(() {});
        },
        child: FContainer(
          padding: const EdgeInsets.all(15),
          child: _buildGroupTitleRow(
            e.itemMetaGroupTitle.toString(),
            e.isExpend!,
          ),
        ),
      ),
      e.isExpend! ? _buildGroupBody(e) : FContainer(),
    ];
  }

  FWidget _buildGroupTitleRow(
    String itemMetaGroupTitle,
    bool isExpend,
  ) {
    return FRow(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        FRow(
          children: [
            FText(
              itemMetaGroupTitle,
            ),
            FText(
              getValues(
                itemMetaGroupTitle,
              ),
            ),
          ],
        ),
        isExpend
            ? const FIcon(
                Icons.keyboard_arrow_down_sharp,
              )
            : const FIcon(
                Icons.keyboard_arrow_right,
              ),
      ],
    );
  }

  FWidget _buildGroupBody(ItemMetaGroup e) {
    return FContainer(
      padding: const EdgeInsets.only(left: 15),
      margin: const EdgeInsets.symmetric(vertical: 10),
      child: FWrap(
        spacing: 16.0, // 主轴(水平)方向间距
        runSpacing: 8.0, // 纵轴（垂直）方向间距
        direction: Axis.horizontal,
        crossAlignment: WrapCrossAlignment.center,
        alignment: WrapAlignment.start,
        children: List<FWidget>.generate(
          e.itemMeta.length,
          (index) {
            return widget.chooseMeasureList.contains(
              e.itemMeta[index].name,
            )
                ? FElevatedButton(
                    onPressed: () {
                      removeChooseMeasure(
                        e.itemMeta[index].name!,
                      );
                    },
                    style: const ButtonStyle(),
                    child: FContainer(
                      width: 180,
                      height: 60,
                      child: FCenter(
                        child: FColumn(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [
                            FText(
                              getValues(
                                e.itemMeta[index].name!,
                              ),
                              style: const TextStyle(
                                color: Colors.white,
                              ),
                            ),
                            FText(
                              e.itemMeta[index].name!,
                              style: const TextStyle(
                                color: Colors.white,
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),
                  )
                : FElevatedButton(
                    onPressed: () {
                      addChooseMeasure(
                        e.itemMeta[index].name!,
                      );
                    },
                    style: ButtonStyle(
                      backgroundColor: MaterialStateProperty.all(
                        Colors.white,
                      ),
                    ),
                    child: FContainer(
                      width: 180,
                      height: 60,
                      child: FCenter(
                        child: FColumn(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [
                            FText(
                              getValues(
                                e.itemMeta[index].name!,
                              ),
                              style: const TextStyle(
                                color: Colors.black,
                              ),
                            ),
                            FText(
                              e.itemMeta[index].name!,
                              style: const TextStyle(
                                color: Colors.black,
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),
                  );
          },
        ),
      ),
    );
  }
}
