import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/view/measure/measure_config/measure_configuation_page.dart';
import 'package:fis_measure/view/measure/measure_config/widgets/measure_configuration_style.dart';
import 'package:fis_theme/theme.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

/// 下拉选择的类
class MeasureSelectModel {
  MeasureSelectModel({required this.name, required this.code});
  final String name;
  final String code;
}

enum TagEnum {
  MeasureTool,
  NodesTool,
}

/// 工具箱 样式配置之类
class LeftSiderHold extends StatelessWidget implements FWidget {
  const LeftSiderHold({Key? key, this.ifHideConfig = false}) : super(key: key);
  final bool ifHideConfig; // 是否隐藏测量项配置入口

  @override
  FWidget build(BuildContext context) {
    return FContainer(
      width: 300,
      padding: const EdgeInsets.symmetric(
        horizontal: 10,
        vertical: 15,
      ),
      child: FColumn(
        key: UniqueKey(),
        mainAxisSize: MainAxisSize.max,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          FRow(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              // 工具箱文案
              FText(
                i18nBook.measure.toolbox.t,
                style: const TextStyle(
                  color: Colors.white,
                ),
              ),
              _LeftSiderHoldAllStyleConfig(ifHideConfig: ifHideConfig),
            ],
          ),
          _LeftSiderTabBar(),

          // _LeftSelectInput(),
          // _LeftPulldown(),
        ],
      ),
    );
  }
}

/// 测量样式 和配置
class _LeftSiderHoldAllStyleConfig extends StatelessWidget implements FWidget {
  const _LeftSiderHoldAllStyleConfig({Key? key, this.ifHideConfig = false})
      : super(key: key);
  final bool ifHideConfig; // 是否隐藏测量项配置入口
  @override
  FWidget build(BuildContext context) {
    return FRow(
      children: [
        FTextButton(
          onPressed: () {
            Get.dialog(
              PatternDialog(),
            );
          },
          child: FText(
            i18nBook.measure.style.t,
            style: TextStyle(
              color: FTheme.ins.colorScheme.primary,
            ),
          ),
        ),
        if (!ifHideConfig) ...[
          FTextButton(
            onPressed: () {
              Get.dialog(
                const MeasureConfigurationPage(),
              );
            },
            child: FText(
              i18nBook.measure.config.t,
              style: TextStyle(
                color: FTheme.ins.colorScheme.primary,
              ),
            ),
          )
        ]
      ],
    );
  }
}

/// 测量工具 注释工具Tab
class _LeftSiderTabBar extends StatefulWidget implements FWidget {
  @override
  State<_LeftSiderTabBar> createState() => _LeftSiderTabBarState();
}

class _LeftSiderTabBarState extends State<_LeftSiderTabBar> {
  final measureHandler = Get.find<MeasureHandler>();

  @override
  FWidget build(BuildContext context) {
    return FRow(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        _buildTab(
          TagEnum.MeasureTool,
          i18nBook.measure.measureTool.t,
        ),
        _buildTab(
          TagEnum.NodesTool,
          i18nBook.measure.annotationTool.t,
        ),
      ],
    );
  }

  FWidget _buildTab(TagEnum tag, String title) {
    return FTextButton(
      style: ButtonStyle(
        padding: MaterialStateProperty.all(
          const EdgeInsets.all(
            6,
          ),
        ),
      ),
      onPressed: () {
        measureHandler.changedTab = tag;
        setState(() {});
      },
      child: FText(
        title,
        style: TextStyle(
          color: measureHandler.changedTab == tag
              ? FTheme.ins.colorScheme.primary
              : Colors.white,
        ),
      ),
    );
  }
}

class _LeftPulldown extends StatelessWidget implements FWidget {
  @override
  FWidget build(BuildContext context) {
    return FContainer(
      margin: const EdgeInsets.symmetric(vertical: 4),
      child: FSelect<MeasureSelectModel, String>(
        source: [
          MeasureSelectModel(
            name: 'General',
            code: 'General',
          )
        ],
        hintText: i18nBook.measure.selectMeasureItem.t,
        value: 'General',
        clearable: false,
        height: 36,
        width: 400,
        optionLabelExtractor: (data) => data.name,
        optionValueExtractor: (data) => data.code,
        onSelectChanged: (value, index) {},
      ),
    );
  }
}
