import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/interfaces/process/workspace/exam_info.dart';
import 'package:fis_measure/process/layout/configuration.dart';
import 'package:fis_measure/process/workspace/measure_controller.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/view/mobile_view/mobile_measure_main_view.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:flutter/services.dart';

/// 测量主页面
class MobileMeasureMainPage extends StatefulWidget implements FWidget {
  const MobileMeasureMainPage(
      this.token, this.recordCode, this.patientCode, this.remedicalCode,
      {Key? key})
      : super(key: key);

  final String token;
  final String patientCode;
  final String remedicalCode;
  final String recordCode;

  @override
  State<StatefulWidget> createState() => _MobileMeasureMainPageState();
}

class _MobileMeasureMainPageState extends State<MobileMeasureMainPage> {
  /// 数据
  final measureData = Get.find<MeasureDataController>();

  late final application = Get.find<IApplication>();

  // final mouseState = Get.put<IMouseState>(MouseState());

  /// 页面loadding
  bool loaded = false;

  // /// 图片loadding
  late bool imageLoaded = false;

  ///检查图片信息表
  List<ExamImageInfo> examImageInfoList = [];

  late final measureHandler = Get.find<IMeasureHandler>();

  /// 测量控制器
  late MeasureController measureController = Get.put(MeasureController(
    "",
    imagesFetchFunc: (code) async {
      return examImageInfoList;
    },
  ));

  /// 获取测量图片所需的图片组 并且写入控制器中 加载
  void getExamImageInfoList(List<RemedicalInfoDTO> remedicals) async {
    for (var element in remedicals) {
      examImageInfoList.add(
        ExamImageInfo(
          element.terminalImages!.imageUrl!,
          element.terminalImages!.previewUrl!,
        ),
      );
    }

    measureController = Get.put(MeasureController(
      "",
      imagesFetchFunc: (code) async {
        return examImageInfoList;
      },
    ));

    await measureController.load();
    int selectedImageIndex = -1;
    if (examImageInfoList
        .any((element) => element.url == measureData.itemCurrentImage)) {
      ExamImageInfo selectedImage = examImageInfoList.firstWhere(
        (element) => element.url == measureData.itemCurrentImage,
      );
      selectedImageIndex = examImageInfoList.indexOf(selectedImage);
      measureController.examInfo.selectedImageIndex = selectedImageIndex;
    }
  }

  void changeImage(sender, e) {
    imageLoaded = e;
    setState(() {});
  }

  @override
  void initState() {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.landscapeLeft, //全屏时旋转方向，左边
    ]);
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersive);
    SystemChrome.setSystemUIChangeCallback((systemOverlaysAreVisible) async {
      await Future.delayed(const Duration(seconds: 1));
      SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersive);
    });
    measureHandler.onChangeImageLoaded.addListener(changeImage);
    loadLayoutConfig();
    getImageInfo();
    super.initState();
  }

  void getImageInfo() {
    measureData.measureImageData = MeasureImageData(
      patientCode: widget.patientCode,
      recordCode: widget.recordCode,
      remedicalCode: widget.remedicalCode,
    );
  }

  /// 加载图像布局配置
  void loadLayoutConfig() async {
    await LayoutConfiguration.ins.loadData();
    setState(() {
      // 加载图像数据
      _initData();
    });
  }

  void onImageLoaded(Object sender, ExamImageInfo? e) async {
    // measureHandler.changeImageLoaded = true;
    if (!mounted) return;
    final currentImage = measureData.remedicalList.firstWhereOrNull(
      (element) => element.terminalImages!.imageUrl == e!.url,
    );
    if (currentImage != null) {
      /// 获取图片详细信息
      var remedicalInfo = await measureData.getImageInfo(
        currentImage.remedicalCode ?? '',
        widget.token,
      );
      if (remedicalInfo != null) {
        measureData.aiResults = remedicalInfo.diagnosisResult ?? '';

        /// ai 良恶性 判断是否有ai
        measureData.diagnosisConclusion = remedicalInfo.diagnosisConclusion;
        if (remedicalInfo.carotidResult != null) {
        } else {}
        try {
          if (e != null) {
            Future.delayed(const Duration(milliseconds: 100), () {
              measureController.playerController.play();
            });
            setState(() {
              loaded = true;
              measureHandler.changeImageLoaded = false;
            });
          }
        } catch (error) {
          setState(() {
            loaded = true;
            measureHandler.changeImageLoaded = false;
          });
        }
      }
    }
  }

  @override
  void dispose() {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
    ]);
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.manual,
        overlays: SystemUiOverlay.values);
    SystemChrome.setSystemUIChangeCallback((systemOverlaysAreVisible) async {
      // do nothing
    });
    measureHandler.onChangeImageLoaded.removeListener(changeImage);
    super.dispose();
  }

  @override
  FWidget build(BuildContext context) {
    FWidget body;
    if (!loaded) {
      const loadingWidget = FCenter(child: FCircularProgressIndicator());
      body = FRow(
        children: const [
          FExpanded(
            child: loadingWidget,
          ),
        ],
      );
    } else {
      body = FRow(
        children: [
          FExpanded(
            child: FColumn(
              mainAxisSize: MainAxisSize.max,
              children: [
                FExpanded(
                  child: imageLoaded
                      ? const FCenter(
                          child: FCircularProgressIndicator(),
                        )
                      : MobileMeasureMainView(
                          key: ValueKey(measureData.itemCurrentImage),
                        ),
                )
              ],
            ),
          ),
        ],
      );
    }

    return FCenter(
      child: FContainer(
        color: Colors.black,
        child: body,
      ),
    );
  }

  void _initData() async {
    List<RemedicalInfoDTO> remedicals = [];
    loaded = false;

    var value = await measureData.getRemedicalList.call(
      widget.patientCode,
      widget.recordCode,
      widget.token,
    );

    for (var remedicalItemList in value) {
      remedicals.addAll(remedicalItemList.remedicalList ?? []);
    }

    /// 获取样式
    // _getMeasureSystemSetting();

    measureData.remedicalList = remedicals;
    var remedicalInfo =
        await measureData.getImageInfo.call(widget.remedicalCode, widget.token);
    if (remedicalInfo != null) {
      measureData.aiResults = remedicalInfo.diagnosisResult ?? '';

      if (remedicalInfo.terminalImages != null) {
        loaded = true;
        measureData.itemCurrentImage =
            remedicalInfo.terminalImages!.imageUrl ?? '';
        getExamImageInfoList(remedicals);
      }
    }

    measureController.imageLoaded.removeListener(onImageLoaded);
    measureController.imageLoaded.addListener(onImageLoaded);
  }
}
