import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:fis_measure/view/measure/measure_tool.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:vid/us/vid_us_unit.dart';

/// 移动端测量项选择器（继承自 测量项页面）
class MobileMeasureSelector extends LeftSiderSelectMeasure {
  const MobileMeasureSelector({Key? key}) : super(key: key);

  @override
  FState<LeftSiderSelectMeasure> createState() => _MobileMeasureSelector();
}

class _MobileMeasureSelector extends LeftSiderSelectMeasureState {
  /// 写死的移动端测量项
  List<MobileMeasureBtn> mobileMeasureBtnList = [
    MobileMeasureBtn(
      icon: Icons.linear_scale,
      displayName: '距离',
      itemMeta: ItemMeta(
        MeasureTypes.Distance,
        measureType: MeasureTypes.Distance,
        description: MeasureTypes.Distance,
        briefAnnotation: "D",
        outputs: [
          ItemOutputMeta(MeasureTypes.Distance, "Distance", VidUsUnit.cm),
        ],
      ),
    ),
    MobileMeasureBtn(
      icon: Icons.border_style,
      displayName: '周长',
      itemMeta: ItemMeta(
        MeasureTypes.Perimeter,
        measureType: MeasureTypes.Perimeter,
        description: MeasureTypes.Perimeter,
        briefAnnotation: "P",
        outputs: [
          ItemOutputMeta(MeasureTypes.Perimeter, "Perimeter", VidUsUnit.cm),
        ],
      ),
    ),
    MobileMeasureBtn(
      icon: Icons.format_shapes,
      displayName: '面积',
      itemMeta: ItemMeta(
        MeasureTypes.Area,
        measureType: MeasureTypes.Area,
        description: MeasureTypes.Area,
        briefAnnotation: "A",
        outputs: [
          ItemOutputMeta(MeasureTypes.Area, "Area", VidUsUnit.cm2),
        ],
      ),
    ),
  ];

  @override
  FWidget build(BuildContext context) {
    return FSizedBox(
      width: 100,
      height: 320,
      child: FColumn(
          mainAxisAlignment: MainAxisAlignment.center,
          mainAxisSize: MainAxisSize.max,
          children: [
            const FSizedBox(height: 30),
            ...mobileMeasureBtnList.map((e) {
              return _buildMeasureBtn(e.itemMeta, e.itemMeta.name == activeName,
                  e.displayName, e.icon);
            }).toList(),
          ]),
    );
  }

  FWidget _buildMeasureBtn(
      ItemMeta itemMeta, bool ifActive, String displayName, IconData icon) {
    return FInkWell(
      onTap: () {
        HapticFeedback.mediumImpact();
        if (ifActive) {
          changeItem(
            ItemMeta(
              "None",
              description: "None",
              outputs: [
                ItemOutputMeta("None", "None", VidUsUnit.cm2),
              ],
            ),
          );
        } else {
          changeItem(itemMeta);
        }
      },
      child: Container(
        margin: const EdgeInsets.fromLTRB(0, 10, 10, 10),
        padding: const EdgeInsets.fromLTRB(10, 0, 10, 0),
        decoration: BoxDecoration(
          border:
              Border.all(color: ifActive ? Colors.grey : Colors.transparent),
          borderRadius: BorderRadiusGeometry.lerp(
              BorderRadius.circular(10), BorderRadius.circular(10), 10),
        ),
        child: FColumn(
          children: [
            const FSizedBox(
              height: 10,
            ),
            FIcon(
              icon,
              color: ifActive ? Colors.white : Colors.grey,
            ),
            FText(
              displayName,
              style: TextStyle(
                color: ifActive ? Colors.white : Colors.grey,
              ),
            ),
            const FSizedBox(
              height: 10,
            ),
          ],
        ),
      ),
    );
  }
}

class MobileMeasureBtn {
  ItemMeta itemMeta;
  String displayName;
  IconData icon;
  MobileMeasureBtn(
      {required this.itemMeta, required this.displayName, required this.icon});
}
