import 'dart:convert';

import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/view/paint/ai_patint_controller.dart';
import 'package:fis_measure/view/paint/date_structure.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

/// 特征分析
class FeatureAnalysis extends StatelessWidget {
  FeatureAnalysis(this.descriptions, {Key? key}) : super(key: key);

  final List<AIDiagnosisDescription>? descriptions;
  late final aiPatintController = Get.find<AiPatintController>();
  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        border: Border.all(
          color: const Color.fromRGBO(54, 169, 206, 1),
        ),
        borderRadius: BorderRadius.circular(4),
        color: Colors.transparent,
      ),
      child: Column(
        children: [
          Row(
            mainAxisSize: MainAxisSize.max,
            children: [
              Expanded(
                child: Container(
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(4),
                      topRight: Radius.circular(4),
                    ),
                    color: Color.fromRGBO(54, 169, 206, 1),
                  ),
                  padding: const EdgeInsets.symmetric(
                    vertical: 4,
                    horizontal: 8,
                  ),
                  child: Text(
                    i18nBook.measure.featureAnalysis.t,
                    style: const TextStyle(
                      color: Colors.white,
                    ),
                  ),
                ),
              )
            ],
          ),
          Scrollbar(
            isAlwaysShown: true,
            child: GridView.count(
              shrinkWrap: true,
              crossAxisCount: 2,
              childAspectRatio: 2,
              controller: ScrollController(),
              children: descriptions!.map((e) {
                return Container(
                  decoration: BoxDecoration(
                    border: Border.all(
                      color: const Color.fromRGBO(54, 169, 206, 1),
                    ),
                  ),
                  padding: const EdgeInsets.all(4),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        _buildDescriptionType(e.type),
                        style: const TextStyle(
                          color: Color.fromRGBO(54, 169, 206, 1),
                        ),
                      ),
                      Text(
                        _buildDescriptionValue(e.value ?? ''),
                        style: const TextStyle(
                          color: Colors.white,
                          fontSize: 14,
                        ),
                      ),
                    ],
                  ),
                );
              }).toList(),
            ),
          ),
        ],
      ),
    );
  }

  String _buildDescriptionType(
      DiagnosisDescriptionEnum diagnosisDescriptionType) {
    switch (diagnosisDescriptionType) {
      case DiagnosisDescriptionEnum.Shape:
        return i18nBook.measure.shape.t;
      case DiagnosisDescriptionEnum.Orientation:
        return i18nBook.measure.orientation.t;
      case DiagnosisDescriptionEnum.EchoPattern:
        return i18nBook.measure.echoPattern.t;
      case DiagnosisDescriptionEnum.LesionBoundary:
        return i18nBook.measure.lesionBoundary.t;
      case DiagnosisDescriptionEnum.Margin:
        return i18nBook.measure.margin.t;
      case DiagnosisDescriptionEnum.Calcification:
        return i18nBook.measure.calcification.t;

      case DiagnosisDescriptionEnum.LesionSize:
        return i18nBook.measure.lesionSize.t;
      default:
        return '';
    }
  }

  String _buildDescriptionValue(String diagnosisDescriptionValue) {
    if (diagnosisDescriptionValue.length > 50) {
      final diagnosisDescription = jsonDecode(diagnosisDescriptionValue);
      bool horizontalAndVertical =
          diagnosisDescription?['HorizontalLengthInPixel'] /
                  diagnosisDescription?['VerticalLengthInPixel'] >
              1;
      return horizontalAndVertical ? '> 1' : '< 1';
    } else {
      switch (diagnosisDescriptionValue) {
        case 'Oval':
          return i18nBook.measure.oval.t;
        case 'Round':
          return i18nBook.measure.round.t;
        case 'Irregular':
          return i18nBook.measure.irregular.t;
        case 'Parallel':
          return i18nBook.measure.parallel.t;
        case 'NonParallel':
          return i18nBook.measure.nonParallel.t;
        case 'Anechoic':
          return i18nBook.measure.anechoic.t;
        case 'Hypoechoic':
          return i18nBook.measure.hypoechoic.t;
        case 'Isoechoic':
          return i18nBook.measure.isoechoic.t;
        case 'Hyperechoic':
          return i18nBook.measure.hyperechoic.t;
        case 'Complex':
          return i18nBook.measure.complex.t;
        case 'Strongechoic':
          return i18nBook.measure.strongechoic.t;
        case 'AbruptInterface':
          return i18nBook.measure.abruptInterface.t;
        case 'EchogenicHalo':
          return i18nBook.measure.echogenicHalo.t;
        case 'Circumscribed':
          return i18nBook.measure.circumscribed.t;
        case 'NonCircumscribed':
          return i18nBook.measure.nonCircumscribed.t;
        case 'NoCalcifications':
          return i18nBook.measure.noCalcifications.t;
        case 'Macrocalcifications':
          return i18nBook.measure.macrocalcifications.t;
        case 'CalcificationsInMass':
          return i18nBook.measure.calcificationsInMass.t;
        case 'CalcificationsOutOfMass':
          return i18nBook.measure.calcificationsOutOfMass.t;
        default:
          return '';
      }
    }
  }
}
