import 'package:fis_measure/interfaces/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/process/workspace/measure_controller.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import '../controller.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:get/get.dart';
import '../events.dart';

part 'tone_bar.dart';
part 'tone_bar_carotid.dart';

class OperateBar extends StatefulWidget implements FWidget {
  const OperateBar({Key? key, this.ifShowResetBtn = true}) : super(key: key);
  final bool ifShowResetBtn;

  @override
  State<OperateBar> createState() => _VidPlayerControlBoardState();
}

class _VidPlayerControlBoardState extends State<OperateBar> {
  final measure3DViewController = Get.find<Measure3DViewController>();
  bool enableCarotidBar = false;
  @override
  void initState() {
    // TODO: implement initState
    measure3DViewController.updatePlayerMode.addListener(_onModeChanged);
    super.initState();
  }

  @override
  void dispose() {
    // TODO: implement dispose
    measure3DViewController.updatePlayerMode.removeListener(_onModeChanged);
    super.dispose();
  }

  /// 模式改变触发更新
  void _onModeChanged(Object s, MeasureMode mode) {
    switch (mode) {
      case MeasureMode.vidMode:
        setState(() {
          enableCarotidBar = false;
        });
        break;
      case MeasureMode.carotid2DMode:
        setState(() {
          enableCarotidBar = true;
        });
        break;
      case MeasureMode.carotid3DMode:
        setState(() {
          enableCarotidBar = true;
        });
        break;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 300,
      padding: const EdgeInsets.symmetric(
        vertical: 10,
      ),
      child: Center(
        child:
            enableCarotidBar ? _buildCarotidOperateBar() : _buildOperateBar(),
      ),
    );
  }

  Widget _buildOperateBar() {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        _BrightnessToneBar(),
        const SizedBox(height: 10),
        _ContrastToneBar(),
        if (widget.ifShowResetBtn) ...[
          const SizedBox(height: 10),
          _ResetToneButton(),
          const SizedBox(height: 10),
        ]
      ],
    );
  }

  Widget _buildCarotidOperateBar() {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        _SharpnessCarotidToneBar(),
        const SizedBox(height: 10),
        _BrightnessCarotidToneBar(),
        const SizedBox(height: 10),
        _ContrastCarotidToneBar(),
        const SizedBox(height: 10),
        _ResetCarotidToneButton(),
        const SizedBox(height: 10),
      ],
    );
  }
}
