import 'dart:ui' as ui;
import 'package:fis_measure/interfaces/process/calculators/output.dart';
import 'package:fis_measure/interfaces/process/calculators/values.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_feature.dart';
import 'package:fis_measure/interfaces/process/modes/mode.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/process/items/item_feature.dart';
import 'package:fis_measure/values/colors.dart';
import 'package:fis_measure/values/unit_desc.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class MeasureResultPanel extends StatefulWidget {
  const MeasureResultPanel({this.resultFontSize = 16.0, Key? key})
      : super(key: key);
  final double resultFontSize;

  @override
  State<StatefulWidget> createState() => _MeasureResultPanelState();
}

class _MeasureResultPanelState extends State<MeasureResultPanel> {
  final _scrollController = ScrollController();
  final application = Get.find<IApplication>();

  final List<String> lines = [];

  bool get hasOutputs => lines.isNotEmpty;

  @override
  Widget build(BuildContext context) {
    if (!hasOutputs) return const SizedBox();

    final child = Container(
      constraints: const BoxConstraints(
        maxWidth: 200,
        minWidth: 150,
        maxHeight: 400,
      ),
      decoration: BoxDecoration(
        border: Border.all(
          color: MeasureColors.ResultBorder,
          width: 2,
        ),
        borderRadius: BorderRadius.circular(4),
        color: Colors.black.withOpacity(0.7),
      ),
      // padding: const EdgeInsets.only(left: 2),
      child: FThemeWidget(
        data: Theme.of(context).copyWith(
          scrollbarTheme: ScrollbarThemeData(
            thumbColor: MaterialStateProperty.all(
              MeasureColors.ResultScrollbar,
            ),
            crossAxisMargin: 1,
          ),
        ),
        child: FScrollbar(
          controller: _scrollController,
          isAlwaysShown: true,
          radius: ui.Radius.zero,
          child: FSingleChildScrollView(
            controller: _scrollController,
            child: FRepaintBoundary(
              child: FIntrinsicHeight(
                child: FIntrinsicWidth(
                  child: FContainer(
                    margin: const EdgeInsets.all(8),
                    child: FColumn(
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: _buildMainList(),
                    ),
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
    return child;
    return Draggable(
      child: child,
      feedback: Material(
        color: Colors.transparent,
        child: child,
      ),
      childWhenDragging: const SizedBox(),
      onDragEnd: (details) {
        final renderBox = context.findRenderObject()! as RenderBox;
        print(context.mediaQuerySize);
        // Application.resultPanelOffset = renderBox.globalToLocal(details.offset);
      },
      onDragUpdate: (details) {
        // final renderBox = context.findRenderObject()! as RenderBox;
      },
    );
  }

  @override
  void initState() {
    _updateOutputs();
    application.updateRenderReady.addListener(_onMeasureRerenderReady);
    super.initState();
  }

  @override
  void dispose() {
    application.updateRenderReady.removeListener(_onMeasureRerenderReady);
    super.dispose();
  }

  List<FWidget> _buildMainList() {
    final list = <FWidget>[
      const FSizedBox(height: 6),
      _buildTitle(application.applicationName),
      _buildTitleDivider(),
    ];
    for (var line in lines) {
      list.add(_buildLine(line));
    }
    return list;
  }

  FWidget _buildLine(String text) {
    return FText(
      text,
      style: TextStyle(
        fontSize: widget.resultFontSize,
        color: MeasureColors.Primary,
      ),
    );
  }

  FWidget _buildTitle(String title) {
    return FText(
      title,
      style: TextStyle(
        fontSize: widget.resultFontSize - 2,
        color: MeasureColors.Primary,
      ),
    );
  }

  FWidget _buildTitleDivider() {
    return const FDivider(
      color: Colors.white,
      height: 0.5,
      endIndent: 2,
    );
  }

  void _onMeasureRerenderReady(Object sender, void e) {
    if (mounted) {
      setState(() {
        _updateOutputs();
        _autoScrollToBottom();
      });
    }
  }

  void _updateOutputs() {
    lines.clear();
    final features = <IMeasureItemFeature>[];
    for (var item in application.measureItems) {
      if (item.measuredFeatures.isNotEmpty) {
        features.addAll(item.measuredFeatures);
      }
      if (item.feature != null) {
        features.add(item.feature!);
      }
    }
    final idLength = features.length.toString().length;
    for (var feature in features) {
      final strList = _FeatureDescConverter(feature).generate(idLength);
      lines.addAll(strList);
    }
  }

  void _autoScrollToBottom() {
    if (lines.isEmpty) return;
    Future.delayed(const Duration(milliseconds: 200), () {
      if (_scrollController.hasClients) {
        _scrollController.jumpTo(_scrollController.position.maxScrollExtent);
      }
    });
  }
}

class _FeatureDescConverter {
  final IMeasureItemFeature feature;
  _FeatureDescConverter(this.feature);

  String idPlaceStr = ' ';
  String idStr = ' ';

  List<String> generate(int idLength) {
    idPlaceStr = ' '.padRight(idLength, ' ');
    idStr = feature.id.toString().padRight(idLength, ' ');

    final List<String> arr = [];
    final mainLines = _findFeatureLines(feature);
    arr.addAll(mainLines);

    final ref = feature.refItem;
    if (ref is ITopMeasureItem) {
      for (var child in ref.childItems) {
        for (var feature in child.measuredFeatures) {
          final str = _findChildLine(feature);
          if (str != null && str.isNotEmpty) {
            arr.add(str);
          }
        }
        if (child.feature != null) {
          final str = _findChildLine(child.feature!);
          if (str != null && str.isNotEmpty) {
            arr.add(str);
          }
        }
      }
    }

    return arr;
  }

  String? _findChildLine(IMeasureItemFeature feature) {
    if (feature.value == null) return null;

    final value = feature.value!;
    final valueStr = _pickValueStr(value);

    final name = feature.refItem.description;
    final unitStr = UnitDescriptionMap.getDesc(value.meta.unit);
    // final output = feature.refItem.meta.outputs.first;
    // final name =
    //     _findDisplayName(output.description, output.briefAnnotation, false);

    return ' $idPlaceStr $name: $valueStr$unitStr';
  }

  List<String> _findFeatureLines(IMeasureItemFeature feature) {
    final count = feature.values.length;
    if (count == 0) {
      final output = feature.refItem.meta.outputs.first;
      final name =
          _findDisplayName(output.description, output.briefAnnotation, false);
      return ['$idStr $name'];
    }

    List<String> arr = [];
    for (var i = 0; i < count; i++) {
      final val = feature.values[i];
      final meta = val.meta;
      final valueStr = _pickValueStr(val);
      final name =
          _findDisplayName(meta.description, meta.briefAnnotation, false);
      final unitStr = UnitDescriptionMap.getDesc(meta.unit);

      if (i == 0) {
        arr.add('$idStr $name: $valueStr$unitStr');
      } else {
        arr.add('$idPlaceStr $name: $valueStr$unitStr');
      }
    }

    return arr;
  }

  String _pickValueStr(ValueBase value) {
    if (value is FloatValue) {
      return _roundDouble(value.value!, value.meta.fractionalDigits).toString();
    } else if (value is StringValue) {
      return value.value!;
    }
    return '';
  }

  String _findDisplayName(String description, String? briefAnnotation,
      [bool showAnnotation = false]) {
    if (showAnnotation) {
      if (briefAnnotation != null && briefAnnotation.isNotEmpty) {
        return briefAnnotation;
      }
    }
    return description;
  }

  double _roundDouble(double value, [int digits = 2]) {
    final digitsStr = value.toStringAsFixed(digits);
    final result = double.parse(digitsStr);
    return result;
  }
}
